function resetCounters()
    gamesCount = GamesView.getGamesCount()

    if gamesCount <= gamesPerRow then
        columnsCount = gamesCount
    elseif gamesCount <= gamesPerPage then
        columnsCount = gamesPerRow
    else
        columnsCount = gamesPerRow + (gamesCount - gamesPerPage + 1) // 2
    end
end

function init()
    font = Gfx.loadFont("assets/fonts/OpenSans-Regular.ttf", 16)
    winTitleFont = Gfx.loadFont("assets/fonts/OpenSans-Regular.ttf", 22)

    screensize = Gui.getScreenSize()
    if screensize.x == 640 then
        gamesPerRow = 4
    else
        gamesPerRow = 5
    end
    gamesPerPage = 2 * gamesPerRow
    coverWidth = 129
    coverHeight = 180
    selectorBorder = 3
    gridGap = 10
    startGrid = 0
    selectedGame = 0
    columnScroll = 0
    columnsCount = 0
    GamesView.setCoverSize(coverWidth, coverHeight)
    resetCounters()
end

function addGameCB()
    resetCounters()
end

function draw(onFocus)
    if GamesView.getGamesType() == 0 then
        Gfx.print(winTitleFont, 0, 0, "GC games")
    elseif GamesView.getGamesType() == 1 then
        Gfx.print(winTitleFont, 0, 0, "Wii games")
    end

    Gfx.translate(0, 30)
    if onFocus then
        Gfx.print(font, 0, 0, GamesView.getGameName(selectedGame))
    end
    Gfx.translate(0, 20)
    Gfx.translate(selectorBorder, selectorBorder)
    Gfx.translate(-columnScroll * (coverWidth + gridGap), 0)
    for i = 0, (gamesCount - 1) do
        local x = 0
        local y = 0
        if i < gamesPerPage then
            x = (i % gamesPerRow) * (coverWidth + gridGap)
            y = (i // gamesPerRow) * (coverHeight + gridGap)
        else
            x = (gamesPerRow + (i - gamesPerPage) // 2) * (coverWidth + gridGap)
            y = ((i - gamesPerPage) % 2) * (coverHeight + gridGap)
        end

        if i == selectedGame and onFocus then
            Gfx.drawRectangle(x - selectorBorder, y - selectorBorder, coverWidth + 2 * selectorBorder, coverHeight + 2 * selectorBorder, Gfx.RGBA8(0x25, 0xcb, 0xf9, 0xff))
        end
        GamesView.drawGameCover(x, y, i + startGrid)
    end
end

function handleInputs()
    local down = Pad.gendown(0)
    local curColumn = 0
    local curRow = 0

    if selectedGame < gamesPerPage then
        curColumn = selectedGame % gamesPerRow
        curRow = selectedGame // gamesPerRow
    else
        curColumn = gamesPerRow + (selectedGame - gamesPerPage) // 2
        curRow = (selectedGame - gamesPerPage) % 2
    end

    if down.BUTTON_RIGHT then
        if (curColumn == columnScroll + gamesPerRow - 1) and (columnScroll + gamesPerRow < columnsCount) then
            columnScroll = columnScroll + 1
        end
        local nextSelGame = 0
        if selectedGame < gamesPerRow - 1 then
            nextSelGame = selectedGame + 1
        elseif selectedGame == gamesPerRow - 1 then
            nextSelGame = selectedGame + gamesPerRow + 1
        elseif selectedGame < gamesPerPage - 1 then
            nextSelGame = selectedGame + 1
        else
            nextSelGame = selectedGame + 2
        end
        if nextSelGame < gamesCount then
            selectedGame = nextSelGame
        end
    end

    if down.BUTTON_LEFT then
        if (curColumn == columnScroll) and (columnScroll > 0) then
            columnScroll = columnScroll - 1
        end
        if selectedGame > gamesPerPage then
            selectedGame =  selectedGame - 2
        elseif selectedGame == gamesPerPage then
            selectedGame = selectedGame - gamesPerRow - 1
        elseif selectedGame > gamesPerRow then
            selectedGame = selectedGame - 1
        elseif selectedGame == gamesPerRow then
            selectedGame = selectedGame
        elseif selectedGame > 0 then
            selectedGame = selectedGame - 1
        end
    end

    if down.BUTTON_DOWN and curRow == 0 then
        --We don't need to check every range because we know curRow value
        local nextSelGame = 0
        if selectedGame < gamesPerRow then
            nextSelGame = selectedGame + gamesPerRow
        else
            nextSelGame = selectedGame + 1
        end
        if nextSelGame < gamesCount then
            selectedGame = nextSelGame
        end
    end

    if down.BUTTON_UP and curRow == 1 then
        --We don't need to check every range because we know curRow value
        if selectedGame > gamesPerPage then
            selectedGame = selectedGame - 1
        else
            selectedGame = selectedGame - gamesPerRow
        end
    end

    --Boot game if requested
    if down.BUTTON_A then
        GamesView.bootGame(selectedGame)
    end

    --Show game config if requested
    if down.BUTTON_B then
        GamesView.showGameConfig(selectedGame)
    end
end

function getDimensions()
    local w = gamesPerRow * coverWidth + (gamesPerRow - 1) * gridGap + 2 * selectorBorder
    local h = 464 --2 * coverHeight + gridGap + 2 * selectorBorder + 20
    return w, h
end
