--This handy function was copied from https://github.com/sulai/Lib-Pico8/blob/master/lang.lua
function enum(names, offset)
    offset=offset or 1
    local objects = {}
    local size=0
    for idr,name in pairs(names) do
        local id = idr + offset - 1
        local obj = {
            id=id,       -- id
            idr=idr,     -- 1-based relative id, without offset being added
            name=name    -- name of the object
        }
        objects[name] = obj
        objects[id] = obj
        size=size+1
    end
    objects.idstart = offset        -- start of the id range being used
    objects.idend = offset+size-1   -- end of the id range being used
    objects.size=size
    objects.all = function()
        local list = {}
        for _,name in pairs(names) do
            add(list,objects[name])
        end
        local i=0
        return function() i=i+1 if i<=#list then return list[i] end end
    end
    return objects
end

function myAtan2(y, x) --This always returns an angle in [0; 2pi)
    local angle = math.atan(y, x) + 2 * math.pi
    return angle % (2 * math.pi)
end

function modAngle(angle)
    return angle % (2 * math.pi)
end

function init()
    stickLoHiThreshold = 80
    stickHiLoThreshold = 60
    curSelection = -1
    timeout = 1000 --ms
    blinkPeriod = 250 --ms
    radius = 100
    cx = 10
    cy = 0
    states = enum({"idle", "active", "stickReleased", "selectionMade", "waitForRelease"})
    state = states.idle
    stateTimer = 0

    c_stick_inner = Gfx.loadImage("assets/switcher/c-stick_inner.png")
    select_slice = Gfx.loadImage("assets/switcher/select.png")
    main_wheel = Gfx.loadImage("assets/switcher/wheel.png")

    nIcons = 8
end

function draw(onFocus)
    if state == states.idle or state == states.waitForRelease then
        return
    end

    Gfx.drawImage(main_wheel)
    local angleOffset = math.pi / nIcons
    local angleStep = 2 * math.pi / nIcons

    local w, h = Gfx.getImageSize(c_stick_inner)
    Gfx.drawImage(c_stick_inner, 157 - w // 2 + cx * 16 // 128, 157 - h // 2 - cy * 16 // 128)

    local angle = math.deg(math.floor(modAngle((myAtan2(cy, cx) + angleOffset)) / angleStep) * angleStep)
    if state == states.selectionMade then
        angle = math.deg(curSelection * angleStep)
    end
    Gfx.pushMatrix()
    Gfx.translate(157 + 52 - 16, 157 - 112)
    Gfx.pushMatrix()
    Gfx.rotate(-52 + 16, 112, -angle)
    if state == states.selectionMade then
        if (Time.getms() - stateTimer) % blinkPeriod < blinkPeriod / 2 then
            Gfx.drawImage(select_slice)
        end
    elseif stickRadius > stickLoHiThreshold then
        Gfx.drawImage(select_slice)
    end
    Gfx.popMatrix()
    Gfx.popMatrix()

end

function handleInputs()
    local angleOffset = math.pi / nIcons
    local angleStep = 2 * math.pi / nIcons

    cx, cy = Pad.subStick(0)
    stickRadius = math.sqrt(cx * cx + cy * cy)

    if stickRadius > stickLoHiThreshold and state == states.idle then
        state = states.active
        stateTimer = Time.getms()
        curSelection = math.floor((math.atan(-cy, cx) + angleOffset) / angleStep)
    elseif state == states.active then
        local tempSelection = math.floor(modAngle((myAtan2(cy, cx) + angleOffset)) / angleStep)

        if stickRadius < stickHiLoThreshold then
            stateTimer = Time.getms()
            state = states.stickReleased
        end

        if tempSelection ~= curSelection then
            stateTimer = Time.getms()
            curSelection = tempSelection
        end

        if (Time.getms() - stateTimer) >= timeout then
            stateTimer = Time.getms()
            state = states.selectionMade
        end
    elseif state == states.stickReleased then
        if (Time.getms() - stateTimer) >= timeout then
            state = states.idle
        elseif stickRadius > stickLoHiThreshold then
            state = states.active
            stateTimer = Time.getms()
            curSelection = math.floor(modAngle((myAtan2(cy, cx) + angleOffset)) / angleStep)
        end
    elseif state == states.selectionMade then
        if (Time.getms() - stateTimer) >= timeout then
            stateTimer = Time.getms()
            state = states.waitForRelease
        end
    elseif state == states.waitForRelease then
        if curSelection == 0 then
            Gui.switchToElement("WiiGamesView")
        elseif curSelection == 1 then
            Gui.switchToElement("GCGamesView")
        elseif curSelection == 5 then
            Gui.switchToElement("HomebrewsView")
        end
        if stickRadius < stickHiLoThreshold then
            state = states.idle
        end
    end
end

function getDimensions()
    local w = 314
    local h = 314
    return w, h
end
